#-----------------------------------------------------------------------------
# Eruption
# January 16, 2004
# Based on Weather
# Copyright (c) 2004, David G. Drumright (aka ockham)

# Load the parent object first;  be sure you have plenty of frames
# in your animation, like 300
#-----------------------------------------------------------------------------

from Tkinter import *
root = Tk()

import poser
import random
import math
import string
random.seed()

#-----------------------------------------------------------------------------

global StartWidth
global WidthOfFall
global HeightOfTop
global SteadyHeightInc
global SteadyWidthIinc
global StartFrame
global EndFrame
global NumDrops

StartWidth = 0.3
WidthOfFall = 0.05
HeightOfTop = 1.0
SteadyHeightInc  = 0.08 # Default starting velocity if not given.
SteadyWidthInc  = 0.04 # Default starting velocity if not given.
# Reminder: we start typically upward, let gravity slow us down
# and then pull us downward.

#-----------------------------------------------------------------------------
# Setup main vars inline.

scene = poser.Scene()
scene.SetFrame(0)

SourceProp = scene.CurrentActor()
if not SourceProp.IsProp(): raise "Can't use the selected actor"
#-----------------------------------------------------------------------------
def MakeDrops(HowMany):
    DropName = os.path.normpath("runtime:libraries:props:Shrew:Rock.pp2")
    for i in range(HowMany):
        scene.LoadLibraryProp(DropName)
        NewProp=scene.CurrentActor()
        NewProp.Memorize()
        NewProp.SetParent(SourceProp)
#-----------------------------------------------------------------------------

def RunDrops():
    # Find true ground by getting the World Displacement of the source.
    (WDX,WDY,WDZ)=SourceProp.WorldDisplacement()
    TrueGround = -WDY # Ground as seen from the source.

    # Set up initial state.  Note that this lists all drops, not just the ones
    # we just now brought in.
    InitX=0.0
    InitY=0.0
    InitZ=0.0
    GravStartFrame=0
    DropList = []
    for OneDrop in SourceProp.Children():
        TryName = OneDrop.Name()
        if string.count(TryName,'drop'): # Eliminate anything that isn't a drop
              DropList.append((TryName,InitX,InitY,InitZ,GravStartFrame))

    # Now go through that list and assign positions, evenly distributed around
    # the centerpoint of the source, using the StartWidth global.
    # Set up initial positions as a circle around the centerpoint of the SourceProp.
    # (Use WD for centerpoint.)
    StepsAroundCircle = len(DropList)
    OneAngleStep =  2.0*math.pi/float(StepsAroundCircle)
    Angle=0.0
    for i in range(len(DropList)):
        (OneDropName,dumX,dumY,dumZ,dumG) = DropList[i]

        InitX = math.cos(Angle) *  StartWidth
        InitZ = math.sin(Angle) *  StartWidth
        InitY = WDY + 0.9 * random.random() # Give each drop a random starting height.
        # This is important; otherwise they will all pop up together.

        # Store the real initial values with the drop.
        DropList[i]=(OneDropName,InitX,InitY,InitZ,i) # Use i for GravStart

        # Now put the starting values onto the drop itself.
        OneDrop = scene.Actor(OneDropName)
        OneDrop.ParameterByCode(poser.kParmCodeASCALE).SetValue(0.015 + (random.random()*0.15) )
        # The first ACTUAL position for all the drops is together, inside the fountain.
        # It's up to the SourceProp to make that work; its offset must be
        # placed so that the drops are all inside before the movement begins.

        OneDrop.ParameterByCode(poser.kParmCodeXTRAN).SetValue(0.0)
        OneDrop.ParameterByCode(poser.kParmCodeYTRAN).SetValue(0.15)
        OneDrop.ParameterByCode(poser.kParmCodeZTRAN).SetValue(0.0)

        Angle=Angle+OneAngleStep # Step the angle

    # At each frame, move all drops on Y by the SteadyYinc global, which may be upward
    # or downward or zero, then move them downward and slightly to one side.
    # Also move all steadily outward on both X and Z, in the same direction as initial move.
    # When each bumps the bottom, push it back to its initial position.

    for F in range(StartFrame+1,EndFrame): # The +1 goes with the hiding mentioned above.
        scene.SetFrame(F)
        for i in range(len(DropList)):
            (DropName,InitX,InitY,InitZ,GravStartFrame) = DropList[i]
            OneDrop = scene.Actor(DropName)
            Xtrans= OneDrop.ParameterByCode(poser.kParmCodeXTRAN)
            Ytrans= OneDrop.ParameterByCode(poser.kParmCodeYTRAN)
            Ztrans= OneDrop.ParameterByCode(poser.kParmCodeZTRAN)
            Xscale= OneDrop.ParameterByCode(poser.kParmCodeXSCALE)
            Yscale= OneDrop.ParameterByCode(poser.kParmCodeYSCALE)
            Zscale= OneDrop.ParameterByCode(poser.kParmCodeZSCALE)
            PresentX =  Xtrans.Value()
            PresentY =  Ytrans.Value()
            PresentZ =  Ztrans.Value()

            if PresentY > TrueGround: # Not at ground yet: Move [up/down] by SteadyYinc, outward by
                # SteadyWidthInc, then downward by gravity.
                NewX = PresentX + SteadyWidthInc * InitX  # Using InitX and InitZ as "dir cosines"
                NewZ = PresentZ + SteadyWidthInc * InitZ
                NewY = PresentY + SteadyHeightInc
                # Figure GravInc based on Frames from StartFrame.
                # We need the distance traveled in this frame,
                # not the total distance.  So we figure dist at this frame
                # and dist at previous frame, and subtract.
                if F == GravStartFrame:
                    GravInc = 0.0

                else:
                    # To get T in seconds, take Frame * (Secs / Frame)
                    # Reminder: we don't need to get the actual previous position;
                    # just find where we are in the time interval.
                    T1 = float((F-0) - GravStartFrame) / float(scene.FramesPerSecond())
                    T0 = float((F-1) - GravStartFrame) / float(scene.FramesPerSecond())

                    # Since one Poser Unit is about 8 feet, and G is 32 f/s/s,
                    # we want 4 PU/s/s for G.
                    # This is based on good old Dist= 1/2 * G * Tsquared.
                    Dist1 = 0.5 * 4.0 * T1 * T1 # Dist at this frame, without the steady component
                    Dist0 = 0.5 * 4.0 * T0 * T0 # Dist at prev frame, without the steady component
                    GravInc = Dist1 - Dist0
                NewY = NewY - GravInc
                Xtrans.SetValue(NewX)
                Ytrans.SetValue(NewY)
                Ztrans.SetValue(NewZ)

            else:
                # We've hit the ground: yank it back to init, and rescale
                # so it will look different
                Xtrans.SetValue(InitX)
                Ytrans.SetValue(InitY)
                Ztrans.SetValue(InitZ)
                Xscale.SetValue(0.3 + 0.3*random.random())
                Yscale.SetValue(0.3 + 0.3*random.random())
                Zscale.SetValue(0.3 + 0.3*random.random())
                # Put back the new GravStart
                DropList[i]=(DropName,InitX,InitY,InitZ,F)



#-----------------------------------------------------------------------------
# Next section is the TK loop, which includes a lot of
# boilerplate stuff.
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):

        self.master = master
        master.title("Binford Porta-Cano ERUPTION!!!")

        Label(master, text="Start frame:"    ).grid(row=0, column=0)
        self.StartEntry = Entry(master)
        self.StartEntry.grid(row=0, column=1)

        Label(master, text="End frame:"      ).grid(row=0, column=2)
        self.EndEntry = Entry(master)
        self.EndEntry.grid(  row=0, column=3)

        # buttons for 'modes' and cancel.
        self.buttonGo = Button(master, text="Go", command=self.HandleGo)
        self.buttonGo.grid(row=3, column=0)

        self.buttonNo = Button(master, text="Cancel", command=self.HandleCancel)
        self.buttonNo.grid(row=3, column=3)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)
#------------------------------------------------------------

    def GetFrameControls(self):  # separate to shorten the Go parts
        global StartFrame
        global EndFrame
        global NumDrops
        try:
           StartFrame = int(self.StartEntry.get()) - 1
        except:
           StartFrame = 0
        try:
           EndFrame   = int(self.EndEntry.get()) - 1
        except:
           EndFrame = scene.NumFrames()-1

#------------------------------------------------------------

    def HandleGo(self):
        self.GetFrameControls()
        MakeDrops(50)
        RunDrops()
        self.master.destroy()

#------------------------------------------------------------

    def HandleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the Win loop, which will be broken by any button.
#------------------------------------------------------------

app = App(root, '')
root.mainloop()

#------------------------------------------------------------
